from django.conf import settings

# These countries will be prioritised in the search
# for a matching timezone. Consider putting your
# app's most popular countries first.
# Defaults to top Internet using countries.
TZ_DETECT_COUNTIRES = getattr(settings, 'TZ_DETECT_COUNTIRES', ('CN', 'US', 'IN', 'JP', 'BR', 'RU', 'DE', 'FR', 'GB'))



from django.utils import timezone


class TimezoneMiddleware(object):
    def process_request(self, request):
        tz = request.session.get('detected_tz')
        if tz:
            # `request.timezone_active` is used in the template
            # tag to detect if the timezone has been activated
            request.timezone_active = True
            timezone.activate(tz)
        else:
            timezone.deactivate()



from django.http import HttpResponse
from django.views.generic import View

from tz_detect.utilities import offset_to_timezone


class SetOffsetView(View):
    http_method_names = ['post']

    def post(self, request, *args, **kwargs):
        offset = request.POST.get('offset', None)
        if not offset:
            return HttpResponse("No 'offset' parameter provided", status_code=400)

        try:
            offset = int(offset)
        except ValueError:
            return HttpResponse("Invalid 'offset' value provided", status_code=400)

        tz = offset_to_timezone(int(offset))
        request.session['detected_tz'] = tz

        return HttpResponse("OK")



from datetime import datetime
from itertools import chain

import pytz

from tz_detect.settings import TZ_DETECT_COUNTIRES


def get_prioritized_timezones():
    def tz_gen():
        for c in TZ_DETECT_COUNTIRES:
            yield pytz.country_timezones(c)
        yield pytz.common_timezones

    return chain.from_iterable(tz_gen())


def offset_to_timezone(offset):
    """Convert a minutes offset (JavaScript-style) into a pytz timezone"""
    clostest_tz = None
    clostest_delta = 1440

    # JS offsets are flipped and can be negative, so
    # unflip and put into range 0 - 1440
    user_offset = (offset * -1)
    user_offset = (user_offset + 1440) % 1440

    for tz_name in get_prioritized_timezones():
        tz = pytz.timezone(tz_name)
        tz_offset = tz.utcoffset(datetime.now()).seconds / 60
        delta = tz_offset - user_offset
        if abs(delta) < abs(clostest_delta):
            clostest_tz = tz
            clostest_delta = delta
            if delta == 0:
                break

    return clostest_tz



from django import template
from django.conf import settings

register = template.Library()


@register.inclusion_tag('tz_detect/detector.html', takes_context=True)
def tz_detect(context):
    return {
        'show': not hasattr(context['request'], 'timezone_active'),
        'STATIC_URL': settings.STATIC_URL,
        'DEBUG': settings.DEBUG,
    }



#!/usr/bin/env python

from os.path import exists
try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

from setuptools import find_packages
from tz_detect import __version__

setup(
    name='django-tz-detect',
    version=__version__,
    author='Adam Charnock',
    author_email='adam@adamcharnock.com',
    packages=find_packages(),
    package_dir={'tz_detect': 'tz_detect'},
    package_data={
        'tz_detect': [
            'templates/tz_detect/*',
            'static/tz_detect/js/*',
        ]
    },
    include_package_data=True,
    # Any executable scripts, typically in 'bin'. E.g 'bin/do-something.py'
    scripts=[],
    # REQUIRED: Your project's URL
    url='https://github.com/adamcharnock/django-tz-detect',
    license='MIT',
    description='Automatic user timezone detection for django',
    long_description=open('README.rst').read() if exists("README.rst") else "",
    # Any requirements here, e.g. "Django >= 1.1.1"
    install_requires=[
        'django>=1.4',
        'pytz',
    ],
)
